package com.swarmcast.fec;

/**
 * Class definition for FECCodeFactory's.
 *
 * (c) Copyright 2000 Justin F. Chapweske
 * 
 * @author Justin Chapweske (orasis@acm.org)
 */
public abstract class FECCodeFactory {

    protected static FECCodeFactory def;

    protected FECCodeFactory() {}

    /**
     * @return An FECCode for the appropriate <code>k</code> and <code>n</code>
     * values.
     */
    public abstract FECCode createFECCode(int k, int n);

    /**
     * @return The default FECCodeFactory which is defined by the property
     * "com.swarmcast.fec.defaultcodefactoryclass".  If this property is
     * not defined then DefaultFECCodeFactory will be used by default.
     */
    public synchronized static FECCodeFactory getDefault() {
        if (def == null) {
            try {
                Class clazz = Class.forName
                    (System.getProperty
                     ("com.swarmcast.fec.defaultcodefactoryclass",
                      "com.swarmcast.fec.DefaultFECCodeFactory"));
                def = (FECCodeFactory) clazz.newInstance();
            } catch (Exception e) {
                // krunky structure, but the easiest way to deal with the 
                // exception.
                def = new DefaultFECCodeFactory();
            }
        }
        return def;
    }
}
